import { contracts, baseUnits, amountFromBaseUnits, displayAmount, } from "@wormhole-foundation/sdk-base";
import { isNative, isTokenId, nativeTokenId, } from "@wormhole-foundation/sdk-definitions";
import { TokenTransfer } from "../../protocols/tokenTransfer";
import { TransferState } from "../../types";
import { AutomaticRoute } from "../route";
export class AutomaticTokenBridgeRoute extends AutomaticRoute {
    NATIVE_GAS_DROPOFF_SUPPORTED = true;
    static meta = {
        name: "AutomaticTokenBridge",
    };
    static supportedNetworks() {
        return ["Mainnet", "Testnet"];
    }
    // get the list of chains this route supports
    static supportedChains(network) {
        if (contracts.tokenBridgeRelayerChains.has(network)) {
            return contracts.tokenBridgeRelayerChains.get(network);
        }
        return [];
    }
    // get the list of source tokens that are possible to send
    static async supportedSourceTokens(fromChain) {
        const atb = await fromChain.getAutomaticTokenBridge();
        const registered = await atb.getRegisteredTokens();
        return [
            nativeTokenId(fromChain.chain),
            ...registered.map((v) => {
                return { chain: fromChain.chain, address: v };
            }),
        ];
    }
    // get the liist of destination tokens that may be recieved on the destination chain
    static async supportedDestinationTokens(sourceToken, fromChain, toChain) {
        return [
            nativeTokenId(toChain.chain),
            await TokenTransfer.lookupDestinationToken(fromChain, toChain, sourceToken),
        ];
    }
    static isProtocolSupported(chain) {
        return chain.supportsAutomaticTokenBridge();
    }
    getDefaultOptions() {
        return { nativeGas: 0.0 };
    }
    async isAvailable() {
        const atb = await this.request.fromChain.getAutomaticTokenBridge();
        if (isTokenId(this.request.source.id)) {
            return await atb.isRegisteredToken(this.request.source.id.address);
        }
        return true;
    }
    async validate(params) {
        try {
            const options = params.options ?? this.getDefaultOptions();
            const { destination } = this.request;
            let nativeGasPerc = options.nativeGas ?? 0.0;
            if (nativeGasPerc > 1.0 || nativeGasPerc < 0.0)
                throw new Error("Native gas must be between 0.0 and 1.0 (0% and 100%)");
            // If destination is native, max out the nativeGas requested
            if (destination && isNative(destination.id.address) && nativeGasPerc === 0.0)
                nativeGasPerc = 1.0;
            const validatedParams = {
                amount: params.amount,
                options: { ...params.options, nativeGas: nativeGasPerc },
                normalizedParams: await this.normalizeTransferParams(params),
            };
            return { valid: true, params: validatedParams };
        }
        catch (e) {
            return { valid: false, params, error: e };
        }
    }
    async normalizeTransferParams(params) {
        const amount = this.request.parseAmount(params.amount);
        const inputToken = isNative(this.request.source.id.address)
            ? await this.request.fromChain.getNativeWrappedTokenId()
            : this.request.source.id;
        const atb = await this.request.fromChain.getAutomaticTokenBridge();
        const fee = await atb.getRelayerFee(this.request.from.address, this.request.to, inputToken.address);
        // Min amount is fee + 5%
        const minAmount = (fee * 105n) / 100n;
        if (baseUnits(amount) < minAmount) {
            throw new Error(`Minimum amount is ${displayAmount({
                amount: minAmount.toString(),
                decimals: amount.decimals,
            })}`);
        }
        const transferableAmount = baseUnits(amount) - fee;
        const { destination } = this.request;
        const options = params.options ?? this.getDefaultOptions();
        let nativeGasPerc = options.nativeGas ?? 0.0;
        // If destination is native, max out the nativeGas requested
        if (destination && isNative(destination.id.address) && nativeGasPerc === 0.0)
            nativeGasPerc = 1.0;
        if (nativeGasPerc > 1.0 || nativeGasPerc < 0.0) {
            throw new Error("Native gas must be between 0.0 and 1.0 (0% and 100%)");
        }
        // Determine nativeGas
        let nativeGasAmount = 0n;
        if (nativeGasPerc > 0) {
            // TODO: currently supporting 2 decimals of the percentage requested
            const scale = 10000;
            const scaledGas = BigInt(options.nativeGas * scale);
            nativeGasAmount = (transferableAmount * scaledGas) / BigInt(scale);
        }
        return {
            fee: amountFromBaseUnits(fee, this.request.source.decimals),
            amount,
            nativeGasAmount: amountFromBaseUnits(nativeGasAmount, this.request.source.decimals),
        };
    }
    async quote(params) {
        return this.request.displayQuote(await TokenTransfer.quoteTransfer(this.request.fromChain, this.request.toChain, this.toTransferDetails(params)));
    }
    async initiate(signer, params) {
        const transfer = this.toTransferDetails(params);
        const txids = await TokenTransfer.transfer(this.request.fromChain, transfer, signer);
        const msg = await TokenTransfer.getTransferMessage(this.request.fromChain, txids[txids.length - 1].txid);
        return {
            from: transfer.from.chain,
            to: transfer.to.chain,
            state: TransferState.SourceFinalized,
            originTxs: txids,
            attestation: { id: msg },
        };
    }
    async *track(receipt, timeout) {
        yield* TokenTransfer.track(this.wh, receipt, timeout, this.request.fromChain, this.request.toChain);
    }
    toTransferDetails(params) {
        const transfer = {
            automatic: true,
            from: this.request.from,
            to: this.request.to,
            amount: baseUnits(params.normalizedParams.amount),
            token: this.request.source.id,
            nativeGas: baseUnits(params.normalizedParams.nativeGasAmount),
        };
        return transfer;
    }
}
//# sourceMappingURL=automatic.js.map